from decimal import *

cdigits = 5
scale = 100000 # initial values

def setprecision(computedigits):
  global cdigits,scale
  cdigits = computedigits
  scale = 10**computedigits

def length (p1,p2):
    """Euclidean distance between two points
    p1 and p2 are points with integer coordinates.
    The results is an integer interval."""
    l_squared = (p2[0]-p1[0])**2 + (p2[1]-p1[1])**2

    # obtain an initial approximation using the
    # square root operation of the decimal package
    oldprec = getcontext().prec
    getcontext().prec = len(str(l_squared))/2+2
    a = int(Decimal(l_squared).sqrt().quantize(Decimal(1),
                                               rounding=ROUND_FLOOR))
    getcontext().prec = oldprec # restore the old precision

    # correct and verify the result.
    # The final result will be correct regardless of how good
    # the initial approximation was (although it may take longer).
    while a**2 > l_squared:      a -= 1
    while (a+1)**2 <= l_squared: a += 1
    if a*2 < l_squared: return Interval_integer(a,a+1)
    else:               return Interval_integer(a,a)
  
def decimal_to_scaled_int(s):
    return int(s*scale)
  
def print_scaled_int(n,ndigits=None):
    """print integer with decimal point inserted
       "ndigits" digits from the right end"""
    if n<0:
        sign = "-"
        n = -n
    else:
        sign = ''
    if ndigits==None: ndigits=cdigits # default
    return sign + ("%d.%0"+str(ndigits)+"d") % divmod(n, 10**ndigits)

class Interval_integer(object):
  """ represents an interval [a,b] with a<=b """
  def __init__(self,a,b=None):
    self.a = a
    if b==None:
        self.b = a
    else:
        self.b = b

  def __add__(self,y):
    if isinstance(y, (int,long)):
      return Interval_integer(self.a + y, self.b + y)
    else: # assume y is an Interval_integer:
      return Interval_integer(self.a + y.a, self.b + y.b)

  def quantize(self,ndigits):
    """try to find a unique value which is guaranteed
    to be the rounded value of the interval"""
    scalehalf = 10**ndigits/2
    rounded_value = (self.a/scalehalf + 1)/2
    val = rounded_value*2*scalehalf
    if val-scalehalf < self.a and val+scalehalf > self.b:
        return rounded_value
    print self, "cannot be rounded uniquely to",
    print ndigits, "fewer digits."
    raise ValueError

  def quantize_and_print(self,ndigits,initialdigits=None):
    """print the unique value which is the rounded value
    of the interval with "ndigits" remaining digits, if it exists"""
    if initialdigits==None: initialdigits=cdigits # default
    return print_scaled_int(
                  self.quantize(initialdigits-ndigits), ndigits)
  
  def __str__(self):
    return "["+str(self.a)+","+str(self.b)+"]"
  def __repr__(self):
    return "Interval("+repr(self.a)+","+repr(self.b)+")"

def keep_best(l):
  """keeps a list that is guaranteed to contain the smallest
     value from a list of intervals"""
  threshold = min(x.b for x in l)
  return [ x for x in l if x.a <= threshold ]

