# checks presence of one ot two "terminal" edges in thin wire

from basic_routines import map_coordinates
import arithmetic
from arithmetic import print_scaled_int, decimal_to_scaled_int
import mwt_polygon

from pieces import W_upper, W_lower, W_point_x, W_point_y, W_point_z
# W_upper and W_lower form the set W.

def check_W_cases():
   mindiff = 999 * arithmetic.scale # infinity

   for i in range(1,7):
     for j in range(i,7): #  for 1<=i<=j<=6:

       p = W_lower[i:len(W_lower)-j] + W_upper
       i_x = p.index(W_point_x)
       i_y = p.index(W_point_y)
       i_z = p.index(W_point_z)
       excl = [(i_x,i_y),(i_x,i_z)] # critical edges

       pcalc = map_coordinates(decimal_to_scaled_int, p)

       # First, compute optimum solution without restriction
       elist = mwt_polygon.mwt(pcalc)

       for (result,sol,degenerate) in elist:
          # convert list of triplets to list
          # of pairs representing edges:
          sole = [(x,y) for [x,y,z] in sol] + \
                 [(y,z) for [x,y,z] in sol]
          if not((i_x,i_y) in sole or (i_x,i_z) in sole or
                 (i_y,i_x) in sole or (i_z,i_x) in sole):
              print "incorrect:",i,j,result,sol,degenerate
              print ("optimal solution does not contain" ,
                        i_x,i_y, "or", i_x,i_z)
              raise StandardError
          if degenerate:
              print "DEGENERATE SOLUTION:",i,j,result,sol,degenerate
              raise StandardError

       # Now, compute optimum solution with the two edges excluded:
       elist2 = mwt_polygon.mwt(pcalc, excluded=excl)

       # consider just the first solution from the list:
       diff = elist2[0][0].a - max(result.b for (result,s,f) in elist)
       mindiff = min(mindiff,diff)

       print "Case v%d, v%d': difference =" % (i,j)
       print print_scaled_int(diff)
       print " "*10+"Best solution value without terminal edges:"
       print_scaled_interval(elist2[0][0])
       print "          There is/are",len(elist),
       print    "best solution(s) without restriction:"
       for result,sol,f in elist: print_scaled_interval(result)
       print
   print "Minimum overall difference =", print_scaled_int(mindiff)

def print_scaled_interval(x):
 print " "*14+"[%s,%s]"%(print_scaled_int(x.a), print_scaled_int(x.b))

print "Program check_W.py. Check terminal edges xy and xz in W"
arithmetic.setprecision(5) # 5 digits are sufficient.
check_W_cases()
